Module: StoryTellerCli

Defined in:
lib/story_teller/version.rb

Overview

Copyright Nels Nelson 2008-2026 but freely usable (see license)

This file is part of the StoryTeller.

The StoryTeller is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The StoryTeller is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the StoryTeller. If not, see <www.gnu.org/licenses/>.

Constant Summary collapse

VERSION =
'1.1.3'.freeze